# Management of Access Control Lists in Zope.
# The lists are derived from a DatabaseConnector.
# Copyright (C) 2000 Shane Hathaway
# E-mail: shathaway@earthling.net
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
'''User.py'''

import Globals
from AccessControl.User import BasicUser, domainSpecMatch
import Acquisition
from Persistence import Persistent
from zLOG import LOG, INFO, ERROR, WARNING

def _debug(msg): print msg
#def _debug(msg): pass

class User (BasicUser, Acquisition.Implicit, Persistent):
    '''Objects of this type are created for each user
    made available through GroupACL.
    '''

    def __init__(self, ui):
        '''Init from the database.'''
        self.login = ui.login
        self.name = ui.name
        self._v_groupNames = list(ui.groupNames)
        self._v_standardRoles = list(ui.roles)
        self._v_domains = list(ui.domains)

    def _importUserInfo(self, ui):
        '''Imports data from the database.'''
        # This will only be called when a user's info
        # has been customized in a GroupACL, which is
        # to be considered a rare occurrence.
        if self.login != ui.login:
            self.login = ui.login
        if self.name != ui.name:
            self.name = ui.name
        self._v_groupNames = list(ui.groupNames)
        self._v_standardRoles = list(ui.roles)
        self._v_domains = list(ui.domains)

    def getUserName(self):
        return self.login

    def getGroupNames(self):
        '''Returns the group names assigned by ACLManager.'''
        if hasattr(self, '_v_groupNames'):
            return self._v_groupNames
        return []

    def getRoles(self):
        '''Returns a list of roles assigned to this user.'''
        if hasattr(self, '_v_standardRoles'):
            return self._v_standardRoles
        else:
            # No roles assigned.
            return ['Anonymous',]

    def getDomains(self):
	'''Returns a list of domains this user can log in from.'''
        if hasattr(self, '_v_domains'):
            return self._v_domains
        return []

    def authenticate(self, password, request):
        '''Authenticates a user.'''
        groupACL = self.aq_parent
	if not self.has_permission('Can Login and Logout', groupACL):
            # Check whether anonymous users can log in.  If so,
            # allow anyone to log in.
            anon = groupACL._nobody.__of__(groupACL)
            if not anon.has_permission('Can Login and Logout', groupACL):
                LOG('GroupACL', INFO, \
                    '%s: User %s does not have the ' \
                    '"Can Login and Logout" permission.' % \
                    (groupACL.getTraversalPath(), self.login))
                return 0

	domains = self.getDomains()
	if domains and not domainSpecMatch(domains, request):
	    return 0

        userdb = groupACL.userdb()
        if userdb is None or not hasattr(userdb, 'checkPassword'):
            return 0
        if userdb.checkPassword(self.login, password):
            return 1
        # Note that it would be incorrect to log failed attempts here.
        return 0

Globals.default__class_init__(User)
